LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY Nexys4Joystick IS
  PORT (
    -- les 16 switchs
    swt : IN STD_LOGIC_VECTOR (15 DOWNTO 0);
    -- les 5 boutons noirs
    btnC, btnU, btnL, btnR, btnD : IN STD_LOGIC;
    -- horloge
    mclk : IN STD_LOGIC;
    -- les 16 leds
    led : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
    -- les anodes pour sélectionner les afficheurs 7 segments à utiliser
    an : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
    -- valeur affichée sur les 7 segments (point décimal compris, segment 7)
    ssg : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
    -- joystick, sur JB ligne basse
    joy_miso : IN STD_LOGIC;
    joy_mosi : OUT STD_LOGIC;
    joy_sclk : OUT STD_LOGIC;
    joy_ss : OUT STD_LOGIC
  );
END Nexys4Joystick;

ARCHITECTURE synthesis OF Nexys4Joystick IS

  --include MasterJoystick
  COMPONENT MasterJoystick IS
    PORT (
      rst : IN STD_LOGIC;
      clk : IN STD_LOGIC;
      en : IN STD_LOGIC;
      joy_leds : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
      miso : IN STD_LOGIC;
      ss : OUT STD_LOGIC;
      sclk : OUT STD_LOGIC;
      mosi : OUT STD_LOGIC;
      coord_x : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
      coord_y : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
      button_1 : OUT STD_LOGIC;
      button_2 : OUT STD_LOGIC;
      button_joystick : OUT STD_LOGIC;
      busy : OUT STD_LOGIC
    );
  END COMPONENT;
  --end include

  --include diviseurClk
  COMPONENT diviseurClk IS
    -- facteur : ratio entre la fréquence de l'horloge origine et celle
    --           de l'horloge générée
    --  ex : 100 MHz -> 1Hz : 100 000 000
    --  ex : 100 MHz -> 1kHz : 100 000
    GENERIC (facteur : NATURAL);
    PORT (
      clk, reset : IN STD_LOGIC;
      nclk : OUT STD_LOGIC);
  END COMPONENT;
  --end include

  --include All7Segments
  COMPONENT All7Segments IS
    PORT (
      clk : IN STD_LOGIC;
      reset : IN STD_LOGIC;
      e0 : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
      e1 : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
      e2 : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
      e3 : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
      e4 : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
      e5 : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
      e6 : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
      e7 : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
      an : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
      ssg : OUT STD_LOGIC_VECTOR (7 DOWNTO 0));
  END COMPONENT;
  --end include

  SIGNAL coord_x : STD_LOGIC_VECTOR (9 DOWNTO 0);
  SIGNAL coord_y : STD_LOGIC_VECTOR (9 DOWNTO 0);
  SIGNAL er_busy : STD_LOGIC;

  SIGNAL sclk : STD_LOGIC;
  SIGNAL rst : STD_LOGIC := NOT btnC;
BEGIN

  Inst_MasterJoyStick : MasterJoystick PORT MAP(
    rst => rst,
    clk => sclk,
    busy => led(4),
    coord_x => coord_x,
    coord_y => coord_y,
    button_1 => led(1),
    button_2 => led(2),
    button_joystick => led(0),
    en => swt(0),
    joy_leds => "100000" & swt(2) & swt(1),
    miso => joy_miso,
    mosi => joy_mosi,
    sclk => joy_sclk,
    ss => joy_ss
  );

  -- convention afficheur 7 segments 0 => allumé, 1 => éteint
  ssg <= (OTHERS => '1');
  -- aucun afficheur sélectionné
  an(7 DOWNTO 0) <= (OTHERS => '1');
  -- 16 leds éteintes
  led(15 DOWNTO 0) <= (OTHERS => '0');

  -- connexion du (des) composant(s) avec les ports de la carte
  Inst_All7Segments : All7Segments PORT MAP(
    clk => mclk,
    reset => rst,
    e0 => coord_x(3 DOWNTO 0),
    e1 => coord_x(7 DOWNTO 4),
    e2 => "00" & coord_x(9 DOWNTO 8),
    e3 => "0000",
    e4 => coord_y(3 DOWNTO 0),
    e5 => coord_y(7 DOWNTO 4),
    e6 => "00" & coord_y(9 DOWNTO 8),
    e7 => "0000",
    an => an,
    ssg => ssg
  );

  Inst_DiviseurClk : diviseurClk GENERIC MAP(
    100) PORT MAP(
    clk => mclk,
    reset => rst,
    nclk => sclk
  );

END synthesis;
