LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY MasterOpl IS
  PORT (
    rst : IN STD_LOGIC;
    clk : IN STD_LOGIC;
    en : IN STD_LOGIC;
    v1 : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
    v2 : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    miso : IN STD_LOGIC;
    ss : OUT STD_LOGIC;
    sclk : OUT STD_LOGIC;
    mosi : OUT STD_LOGIC;
    val_nand : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
    val_nor : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
    val_xor : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
    busy : OUT STD_LOGIC;
    -- debug_state : OUT NATURAL;
    -- debug_er_state : OUT NATURAL;
    er_busy : OUT STD_LOGIC
    -- debug_cpt_wait : OUT NATURAL;
    -- debug_cpt_octet : OUT NATURAL
  );
END MasterOpl;

ARCHITECTURE behavioral OF MasterOpl IS

  TYPE t_etat IS (idle, waiting, byte_sent);

  COMPONENT er_1octet PORT (
    rst : IN STD_LOGIC;
    clk : IN STD_LOGIC;
    en : IN STD_LOGIC;
    din : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
    miso : IN STD_LOGIC;
    sclk : OUT STD_LOGIC;
    mosi : OUT STD_LOGIC;
    dout : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
    busy : OUT STD_LOGIC;
    state : OUT NATURAL
    );
  END COMPONENT;

  SIGNAL etat : t_etat;

  SIGNAL er_en : STD_LOGIC;
  SIGNAL er_dout : STD_LOGIC_VECTOR(7 DOWNTO 0);
  SIGNAL er_din : STD_LOGIC_VECTOR(7 DOWNTO 0);
BEGIN
  relay : er_1octet PORT MAP(
    rst => rst,
    clk => clk,
    busy => er_busy,
    din => er_din,
    mosi => mosi,
    miso => miso,
    dout => er_dout,
    en => er_en,
    sclk => sclk
    -- state => debug_er_state
  );

  PROCESS (clk, rst)
    VARIABLE cpt_wait : NATURAL;
    VARIABLE cpt_octet : NATURAL;
  BEGIN
    IF (rst = '0') THEN
      cpt_octet := 0;
      -- debug_cpt_octet <= cpt_octet;
      er_en <= '0';
      er_din <= (OTHERS => 'U');
      etat <= idle;
      -- debug_state <= 0;
      busy <= '0';
      ss <= '1';
    ELSIF (rising_edge(clk)) THEN
      CASE etat IS
        WHEN idle =>
          -- debug_state <= 1;
          IF (en = '1') THEN
            cpt_wait := 20;
            -- debug_cpt_wait <= cpt_wait;
            cpt_octet := 3;
            -- debug_cpt_octet <= cpt_octet;
            etat <= waiting;
            busy <= '1';
            ss <= '0';
          END IF;

        WHEN waiting =>
          cpt_wait := cpt_wait - 1;
          -- debug_cpt_wait <= cpt_wait;
          IF (cpt_wait = 0) THEN
            IF (cpt_octet = 3) THEN
              er_din <= v1;
            ELSE
              er_din <= v2;
            END IF;

            er_en <= '1';
            etat <= byte_sent;
            -- debug_state <= 2;
          END IF;

        WHEN byte_sent =>
          -- debug_state <= 3;
          er_en <= '0';
          IF (er_en = '0' AND er_busy = '0') THEN
            CASE cpt_octet IS
              WHEN 3 =>
                val_nand <= er_dout;
                cpt_octet := cpt_octet - 1;
                -- debug_cpt_octet <= cpt_octet;
                cpt_wait := 5;
                -- debug_cpt_wait <= cpt_wait;
                etat <= waiting;
              WHEN 2 =>
                val_nor <= er_dout;
                cpt_octet := cpt_octet - 1;
                -- debug_cpt_octet <= cpt_octet;
                cpt_wait := 5;
                -- debug_cpt_wait <= cpt_wait;
                etat <= waiting;
              WHEN 1 =>
                val_xor <= er_dout;
                etat <= idle;
                busy <= '0';
                ss <= '1';
              WHEN OTHERS => NULL;
            END CASE;
          END IF;
      END CASE;
    END IF;
  END PROCESS;
END behavioral;